/* cmus radio driver */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <wiringPi.h>
#include "lcd/lcd.h"

// define buttons and map to WiringPi

// Radio Presets
#define NUM_PRESETS 6
#define PRESET1 10
#define PRESET2 11
#define PRESET3 12
#define PRESET4 13
#define PRESET5 14
#define PRESET6 15

// Autotune button
#define SHOW_STATION 16

// Setup button
#define RESTART_CMUS 21

// Display button
#define TOGGLE_PAUSE 22	

#define CALL_OS_BUFFSIZE 512
char callOSbuff[CALL_OS_BUFFSIZE];

volatile unsigned presetHit[NUM_PRESETS] = {0, 0, 0, 0, 0, 0};
volatile unsigned showStationHit = 0;
volatile unsigned restartCmusHit = 0;
volatile unsigned togglePauseHit = 0;
volatile unsigned radioStopped = 0;

//
// ISRs
//
void preset1ISR(void) {
   presetHit[0]++;
}

void preset2ISR(void) {
   presetHit[1]++;
}

void preset3ISR(void) {
   presetHit[2]++;
}

void preset4ISR(void) {
   presetHit[3]++;
}

void preset5ISR(void) {
   presetHit[4]++;
}

void preset6ISR(void) {
   presetHit[5]++;
}

void showStationISR(void) {
	showStationHit++;
}

void restartCmusISR(void) {
	restartCmusHit++;
}

void togglePauseISR(void) {
	togglePauseHit++;
}

//
// Functions
//

// do a system console command and return stdout
void callOS(char * command) {
char buff[CALL_OS_BUFFSIZE];

	buff[0] = '\0';
	callOSbuff[0] = '\0';
	FILE* cmdPipe = popen(command, "r");		// Send the command
	if (!cmdPipe) {
		strcat(callOSbuff, "FAIL: ");
		strcat(callOSbuff, command);
	} else {
		while(fgets(buff, sizeof(buff), cmdPipe) != NULL){
			strcat(callOSbuff, buff);
		}
    }
	pclose(cmdPipe);
}

// display the radio station name on line 1 of the LCD
char tuningStation[17];
void displayStation() {

  	// show the station
  	lcdPosition(0, 0); 
    lcdPuts(tuningStation);
    delay(5);
    
	// clear the bottom LCD line
  	lcdPosition(0, 1); 
  	lcdPuts("                ");
}

// map preset to stream URL
char streamURL[128];
void getURL(unsigned URLnum) {
	streamURL[0] = '\0';
	switch(URLnum) {
	case 1: strcpy(streamURL, "http://stream-uk1.radioparadise.com/mp3-192");
			strcpy(tuningStation, "Radio Paradise  ");
			break;
	case 2: strcpy(streamURL, "http://188.40.135.197:8388/listen.pls");
			strcpy(tuningStation, "Radio NorthSea  ");
			break;
	case 3: strcpy(streamURL, "http://av.rasset.ie/av/live/radio/gold.m3u");
			strcpy(tuningStation, "RTe Gold        ");
			break;
	case 4: strcpy(streamURL, "http://audio-mp3.ibiblio.org:8000/wcpe.mp3");
			strcpy(tuningStation, "WCPE Classical  ");
			break;
	case 5: strcpy(streamURL, "http://po192.pinguinradio.com:80/");
			strcpy(tuningStation, "Pinguin on Rocks");
			break;
	case 6: strcpy(streamURL, "http://www.listenlive.eu/bbc6music.m3u");
			strcpy(tuningStation, "BBC 6 Music     ");
			break;
	default: printf("gone off end of getURL switch\n");
	}
}

// display the station being tuned and tune to it
void actOnPreset(unsigned presetNum) {
	getURL(presetNum);

  	lcdPosition(0, 0); 
  	lcdPuts("Tuning to ...   ");
  	delay(5);
  	lcdPosition(0, 1); 
  	lcdPuts(tuningStation);
  	delay(5);
  	
	callOSbuff[0] = '\0';
	strcat(callOSbuff, "sudo -u pi cmus-remote -q -c ");
	strcat(callOSbuff, streamURL);
    system(callOSbuff);
	system("sudo -u pi cmus-remote -n");
}

//
// main
//
int main(void) {
char line1[17];
char line2[17];

  // Set up the wiringPi library
  if (wiringPiSetup() < 0) {
      fprintf (stderr, "Unable to setup wiringPi: %s\n", strerror (errno));
      return 1;
  }

  lcdSetup();
  
  // 'splash' screen
  lcdPosition(0, 0); 
  lcdPuts("Fletch's Radio");
  delay(5);
  lcdPosition(0, 1); 
  lcdPuts("v0.2");
  delay(1000);

  // Set up ISRs
  if (wiringPiISR(PRESET1, INT_EDGE_FALLING, &preset1ISR) < 0) {
      fprintf (stderr, "Unable to setup PRESET1 ISR: %s\n", strerror (errno));
      return 1;
  }
  if (wiringPiISR(PRESET2, INT_EDGE_FALLING, &preset2ISR) < 0) {
      fprintf (stderr, "Unable to setup PRESET2 ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(PRESET3, INT_EDGE_FALLING, &preset3ISR) < 0) {
      fprintf (stderr, "Unable to setup PRESET3 ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(PRESET4, INT_EDGE_FALLING, &preset4ISR) < 0) {
      fprintf (stderr, "Unable to setup PRESET4 ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(PRESET5, INT_EDGE_FALLING, &preset5ISR) < 0) {
      fprintf (stderr, "Unable to setup PRESET5 ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(PRESET6, INT_EDGE_FALLING, &preset6ISR) < 0) {
      fprintf (stderr, "Unable to setup PRESET6 ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(SHOW_STATION, INT_EDGE_FALLING, &showStationISR) < 0) {
      fprintf (stderr, "Unable to setup SHOW_STATION ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(RESTART_CMUS, INT_EDGE_FALLING, &restartCmusISR) < 0) {
      fprintf (stderr, "Unable to setup RESTART_CMUS ISR: %s\n", strerror (errno));
      return 1;
  }
  
  if (wiringPiISR(TOGGLE_PAUSE, INT_EDGE_FALLING, &togglePauseISR) < 0) {
      fprintf (stderr, "Unable to setup TOGGLE_PAUSE ISR: %s\n", strerror (errno));
      return 1;
  }
  
  // display the initial radio station, default to preset 1
  // this doesn't work unless the station is manually, once-only, put into cmus' 
  // library, playlist and play queue
   actOnPreset(1);
   displayStation();

  // set up so the station and song are displayed
  // note this overwrites any LCD text written in this code
  system("sudo -u pi cmus-remote -C \"set status_display_program=/home/pi/src/cmusradio/cmus-status-display\"");
  
  // loop checking buttons and displaying station details
  while(1) {
	// check if buttons have been pressed -- only action one button
    if (presetHit[0]) {
		actOnPreset(1);
    } else 
    if (presetHit[1]) {
		actOnPreset(2);
    } else
    if (presetHit[2]) {
		actOnPreset(3);
    } else
    if (presetHit[3]) {
		actOnPreset(4);
    } else
    if (presetHit[4]) {
		actOnPreset(5);
    } else
    if (presetHit[5]) {
		actOnPreset(6);
    } else
    if (showStationHit) {
		displayStation();
    } else
    if (restartCmusHit) {
      	lcdPosition(0, 0); 
		lcdPuts("Fletch's Radio  ");
		delay(5);
  		lcdPosition(0, 1); 
  		lcdPuts("is rebooting ...");
		system("sudo reboot");
    } else
    if (togglePauseHit) {
      	displayStation(); 
      	if (radioStopped) {
  			system("sudo -u pi cmus-remote -p"); // start playing
  			radioStopped = 0;
  		} else {
  			lcdPosition(0, 1); 
  			lcdPuts("is paused ...   ");
			system("sudo -u pi cmus-remote -s");
			radioStopped = 1;
		}
    }
    
    // display the station name if a station has been chosen
    if ((presetHit[0] + presetHit[1] + presetHit[2] + presetHit[3] + presetHit[4] + 
    presetHit[5] + presetHit[6]) > 0) {
    	displayStation();
    }
    
    // clear all button presses
    presetHit[0] = presetHit[1] = presetHit[2] = presetHit[3] = presetHit[4] = 
    presetHit[5] = presetHit[6] = showStationHit = restartCmusHit = 
    togglePauseHit = 0;

    delay(100); // wait 1/10s
  }

  return 0;
}
